/*
 * JBoss, Home of Professional Open Source
 * Copyright 2013, Red Hat, Inc. and/or its affiliates, and individual
 * contributors by the @authors tag. See the copyright.txt in the
 * distribution for a full listing of individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jboss.as.quickstarts.kitchensink.util;

import java.util.logging.Logger;

import javax.enterprise.context.RequestScoped;
import javax.enterprise.inject.Disposes;
import javax.enterprise.inject.Produces;
import javax.enterprise.inject.spi.InjectionPoint;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceUnit;

import org.apache.deltaspike.core.api.config.ConfigProperty;

/**
 * This class uses CDI to alias Java EE resources, such as the persistence context, to CDI beans
 * 
 * <p>
 * Example injection on a managed bean field:
 * </p>
 * 
 * <pre>
 * &#064;Inject
 * private EntityManager em;
 * </pre>
 */
public class Resources {

    /**
     * We are using a non JTA managed persistence context, so we cannot inject an EntityManager with &#064;PersistenceContext,
     * as this would try to inject a container managed EntityManager.
     */
    @PersistenceUnit
    private EntityManagerFactory entityManagerFactory;

    /**
     * Inject a DeltaSpike configuration that can be configured using:
     * 
     * <ul>
     * <li>System properties</li>
     * <li>Environment properties</li>
     * <li>JNDI values</li>
     * <li>Properties file values (apache-deltaspike.properties)</li>
     * </ul>
     */
    @Inject
    @ConfigProperty(name = "jsf.ajax.validation.enabled", defaultValue = "true")
    private Boolean ajaxValidationEnabled;

    @Produces
    @RequestScoped
    protected EntityManager createEntityManager() {
        return this.entityManagerFactory.createEntityManager();
    }

    protected void closeEntityManager(@Disposes EntityManager entityManager) {
        if (entityManager.isOpen()) {
            entityManager.close();
        }
    }

    @Produces
    public Logger produceLog(InjectionPoint injectionPoint) {
        return Logger.getLogger(injectionPoint.getMember().getDeclaringClass().getName());
    }

    @Produces
    @RequestScoped
    public FacesContext produceFacesContext() {
        return FacesContext.getCurrentInstance();
    }

    @Produces
    @Named
    public Boolean getAjaxValidationDisabled() {
        // Here we invert the value since <f:ajax/> tag queries for 'disabled'.
        return !ajaxValidationEnabled;
    }

}
